// T13simDisc.sce
// SIMULATION OF DISCRETE MODEL
//  (multinomial model - controlled coin with memmory)
//         f( y(t) | u(t),y(t-1) ),  y,u=1,2
// Experiments
// - set the parameters to obtain a deterministic model
// - try to extend the model to f(y(t)|u(t),y(t-1),u(t-1)) 
//   and values 1,2,3.
// -------------------------------------------------------------------
exec('SCIHOME/ScIntro.sce',-1), mode(0), getd('functions')

// PARAMETERS OF THE SIMULATION
// model parameter (conditional probabilities)
// u(t) y(t-1)
//  11  12  21 22
// ---------------------------
th= [.2 .6 .9 .3    // y(t)=1
     .8 .4 .1 .7];  // y(t)=2

nd=50;                        // number of steps
u=(.3<rand(1,nd,'u'))+1;      // control variable P(u=1)=.3, P(u=2)=.7
y(1)=1;                       // initial condition for output

// TIME LOOP OF THE SIMULATION
for t=2:nd                    
  i=2*(u(t)-1)+y(t-1);        // row in the model parameter
  y(t)=sum(cumsum(th(:,i))<rand(1,1,'u'))+1; 
                              // generation of the output
end

// RESULTS OF THE SIMULATION
subplot(211),plot(1:nd,u,'g:.')
set(gcf(),"position",[700 100 600 500])
title('Input')
set(gca(),'data_bounds',[0 nd+1 .9 2.1])
subplot(212),plot(1:nd,y,'b:.')
title('Output')
set(gca(),'data_bounds',[0 nd+1 .9 2.1])

