// T21estCont_LS.sce
// ESTIMATION OF 2ND ORDER REGRESSION MODEL
//   least squares estimation (off-line)
// Experiments
// - extend the model order
// -------------------------------------------------------------------
exec('ScIntro.sce',-1), mode(0), getd('functions')

// SIMULATION
// parameters
nd=100;                       // length of data
a=[.4 .2];                    // parameters at y
b=[1 .2 -.5];                 // parameters at u
k=0;                          // constant (model absolute term)
s=.1;                         // noise variance

y(1)=1; y(2)=3;             // initial conditions for output
u=sin(10*%pi*(1:nd)/nd)'+.001*rand(nd,1,'n'); // input

// time loop
th=[a b k]';                  // vector of parameters
for t=3:nd
  // regression vector
  ps=[y(t-1) y(t-2) u(t) u(t-1) u(t-2) 1]';
  // regression model
  y(t)=th'*ps+s*rand(1,1,'n'); 
end

// ESTIMATION
for t=3:nd
  Y(t,1)=y(t);
  X(t,:)=[y(t-1) y(t-2) u(t) u(t-1) u(t-2) 1];
end
th=inv(X'*X)*X'*Y;            // regression coefficients
yp=X*th;                      // prediction (for verification)
r=variance(y-yp);             // noise variance

// Results
disp('Parameter estimates')
th,r

set(scf(1),'position',[800 100 600 400]); 
plot(1:nd,y,1:nd,yp)  // comparison od output and prediction
legend('optput','prediction');
title('Verification of the estimates')
