// T22estCont_B.sce
// ESTIMATION OF 2ND ORDER REGRESSION MODEL
// - Bayesian on-line estimation with statistic update
// Experiments 
// - rewrite the program to a single time loop (on-line estimation)
// -------------------------------------------------------------------
exec('ScIntro.sce',-1), mode(0), getd('functions')

// SIMULATION
// parameters
nd=100;                       // length of data
a=[.4 .2];                    // parameters at y
b=[1 .2 -.5];                 // parameters at u
k=0;                          // constant (model absolute term)
s=.1;                         // noise variance

y(1)=1; y(2)=3;             // initial conditions for output
u=sin(10*%pi*(1:nd)/nd)'+.001*rand(nd,1,'n'); // input

// time loop
th=[a b k]';                  // vector of parameters
for t=3:nd
  // regression vector
  ps=[y(t-1) y(t-2) u(t) u(t-1) u(t-2) 1]';
  // regression model
  y(t)=th'*ps+s*rand(1,1,'n'); 
end

// ESTIMATION
V=1e-8*eye(7,7);              // initial information matrix
for t=3:nd
  psi=[y(t:-1:t-2); u(t:-1:t-2); 1];         // reg. vector
  V=V+psi*psi';                              // updt of information matrix
  Vy=V(1,1); Vyp=V(2:$,1); Vp=V(2:$,2:$);    // divisioning of inf. matrix
  th(:,t)=inv(Vp+1e-8*eye(Vp))*Vyp;          // pt estimates of reg. coefficints
  r(t)=(Vy-Vyp'*inv(Vp+1e-8*eye(Vp))*Vyp)/t; // pt estimates of noise variance
end

// PREDICTION (as evaluation)
thP=th(:,$);                                 // vector of parameters
for t=3:nd
  // regression vector
  ps=[y(t-1) y(t-2) u(t) u(t-1) u(t-2) 1]';
  // regression model
  yp(t)=thP'*ps+sqrt(r($))*rand(1,1,'n'); 
end

// Results
set(scf(1),'position',[50 300 400 400])
for i=1:6
  subplot(6,1,i),plot(th(i,:))
  if i==1, title('Regression coefficients'), end
end
set(scf(2),'position',[50 10 400 200])
plot(r)
title('Noise variance')

set(scf(3),'position',[500 60 800 500])
plot(1:nd,y,1:nd,yp)
title 'Output and its prediction'
