// T22estCont_B2.sce
// ESTIMATION OF 2ND ORDER REGRESSION MODEL
// The model for simulation differs from that for estimation
// i.e. MODEL STRUCTURE MISMATCH is tackled.
// - Bayesian on-line estimation with statistic update
// Experiments 
// - try various input signals - set inp = 1,2,3,4
// - set different noise variance r=0,.1,1,10
// - define other regression coefficients a,b,k
// - you can also change the orders of models for simulation 
//   as well as for estimation (adjust the beginning of loop)
// -------------------------------------------------------------------
exec('ScIntro.sce',-1), mode(0), getd('functions')

// SIMULATION
// parameters
nd=100;                       // length of data
a=[.1 -.8 .3];                // parameters at y
b=[1 .2 -.5 -.8];             // parameters at u
k=0;                          // constant (model absolute term)
s=.01;                        // noise variance
inp=4;                        // selection of input

y(1)=0; y(2)=0; y(3)=0;    // initial conditions for output
                                             // inputs at disposal 
select inp
case 1, u=ones(nd,1);                       // one jump
case 2, u=sin(10*%pi*(1:nd)/nd)';           // several jumps
case 3, u=sign(10*sin(10*%pi*(1:nd)/nd))';  // sine wave
case 4, u=.1*rand(nd,1,'n');                // white noise
end

// time loop
th=[a b k]';                  // vector of parameters
for t=4:nd
  // regression vector
  ps=[y(t-1) y(t-2) y(t-3) u(t) u(t-1) u(t-2) u(t-3) 1]';
  // regression model
  y(t)=th'*ps+s*rand(1,1,'n'); 
end

// ESTIMATION
V=1e-8*eye(5,5);              // initial information matrix
for t=3:nd
  psi=[y(t:-1:t-1); u(t:-1:t-1); 1];         // reg. vector
  V=V+psi*psi';                              // updt of information matrix
  Vy=V(1,1); Vyp=V(2:$,1); Vp=V(2:$,2:$);    // divisioning of inf. matrix
  thE(:,t)=inv(Vp+1e-8*eye(Vp))*Vyp;         // pt estimates of reg. coefficints
  r(t)=(Vy-Vyp'*inv(Vp+1e-8*eye(Vp))*Vyp)/t; // pt estimates of noise variance
end

// PREDICTION (as evaluation)
thP=thE(:,$);                                 // vector of parameters
for t=3:nd
  // regression vector
  ps=[y(t-1) u(t) u(t-1) 1]';
  // regression model
  yp(t)=thP'*ps+sqrt(r($))*rand(1,1,'n'); 
end

// Results
set(scf(1),'position',[50 300 400 400])
nth=size(thE,1);
for i=1:nth
  subplot(nth,1,i),plot(thE(i,:))
  if i==1, title('Regression coefficients'), end
end
set(scf(2),'position',[50 10 400 200])
plot(r)
title('Noise variance')

set(scf(3),'position',[500 60 800 500])
plot(1:nd,y,1:nd,yp)
title 'Output and its prediction'
legend('y','yp');

disp(th','Simulated parameters')
disp(thE(:,$)','Easimated parameters')

disp 'Relative prediction error'
RPE=variance(y-yp)/variance(y)
