// T22estCont_B4.sce
// ESTIMATION OF 2ND ORDER REGRESSION MODEL
// Estimation with REAL DATA (intensities of traffic in Strahov tunnel)
// i.e. MODEL STRUCTURE MISMATCH is tackled.
// - Bayesian on-line estimation with statistic update
// - cannot be done without estimation !!!
// Experiments 
// - change the order of model for estimation
// Result: estimated parameters (better with prediction - T32preCont_Adapt3.sce)
// -------------------------------------------------------------------
exec('ScIntro.sce',-1), mode(0), getd('functions')

ord=5;                        // order of the estimated model
// selection of dataset
k=3;                          // selected day
nz=50;                        // beginning of the day
nd=288;                       // length of the whole one day data

// DATA
dtAll=csvRead('STRAHOV.csv',';');
dt=dtAll((k-1)*288+(nz+1:nd),1);
y=dt(:,1);

// ESTIMATION
nth=ord+2;
V=1e-8*eye(nth,nth);               // initial information matrix
for t=(ord+1):(nd-nz)
  psi=[y(t:-1:t-ord); 1];          // regression vector
  V=V+psi*psi';                              // updt of information matrix
  Vy=V(1,1); Vyp=V(2:$,1); Vp=V(2:$,2:$);    // divisioning of inf. matrix
  thE(:,t)=inv(Vp+1e-8*eye(Vp))*Vyp;         // pt estimates of reg. coefficints
  r(t)=(Vy-Vyp'*inv(Vp+1e-8*eye(Vp))*Vyp)/t; // pt estimates of noise variance
end

// PREDICTION (as evaluation)
thP=thE(:,$);                                 // vector of parameters
for t=(ord+1):(nd-nz)
  // regression vector
    ps=[y(t-1:-1:t-ord); 1];          // regression vector
  // regression model
  yp(t)=thP'*ps+sqrt(r($))*rand(1,1,'n'); 
end

// Results
set(scf(1),'position',[50 300 400 400])
nth=size(thE,1);
for i=1:nth
  subplot(nth,1,i),plot(thE(i,:))
  if i==1, title('Regression coefficients'), end
end
set(scf(2),'position',[50 10 400 200])
plot(r)
title('Noise variance')

set(scf(3),'position',[500 60 800 500])
plot([y,yp])
title 'Output and its prediction'
legend('y','yp');

disp(th','Simulated parameters')
disp(thE(:,$)','Easimated parameters')

disp 'Relative prediction error'
RPE=variance(y-yp)/variance(y)


//// Results
//set(scf(1),'position',[500 60 600 500])
//for i=1:nth-1
//  subplot(nth,1,i),plot(thE(i,:))
//  if i==1, title('Regression coefficients'), end
//end
//set(scf(2),'position',[50 60 400 200])
//plot(r)
//title('Noise variance')
//set(scf(3),'position',[50 360 400 200])
//plot(1:nd-nz,y)
//legend('output');
//title 'Dataset for estimation'
//
//disp(thE(:,$)','Easimated parameters')
