// T23estDisc.sce
// ESTIMATION OF DISCRETE MODEL
//    f(y(t)|u(t),y(t-1)) with y,u from {0,1}
// Experiments
// - change number of values of individual variables
// - increase the model order
// -------------------------------------------------------------------
exec('ScIntro.sce',-1), mode(0), getd('functions')

// SIMULATION
nd=200;                       // number of steps
// parameters of simulation
thS= [.2 .6 .9 .3
      .8 .4 .1 .7];
thU=[.3 .7];
y(1)=1; u(1)=1;               // initial condition for output
// time loop of simulation
for t=2:nd                    
  u(t)=sum(cumsum(thU)<rand(1,1,'u'))+1;
                              // control variable P(u=1)=.3, P(u=2)=.7
  i=2*(u(t)-1)+y(t-1);        // row in the model parameter
  y(t)=sum(cumsum(thS(:,i))<rand(1,1,'u'))+1;  
                              // generation of the output
end

// ESTIMATION
V=zeros(2,4);                 // initial statistics
for t=2:nd
  i=2*(u(t)-1)+y(t-1);        // row of model matrix
  V(y(t),i)=V(y(t),i)+1;      // updt of statistics
  
  for j=1:4                   // point estimates (normalization)
    tht(:,j)=V(:,j)/sum(V(:,j));
  end
  thE(t,:)=tht(1,:);          // remember
end

// PREDICTION
yp(1)=1;
for t=2:nd
  i=2*(u(t)-1)+y(t-1);        // row in the model parameter
  yp(t)=sum(cumsum(thS(:,i))<rand(1,1,'u'))+1;  
end

// Results
set(scf(1),'position',[60 60 600 500])
for i=1:4
  subplot(4,1,i)
  plot(thE(:,i))                                  // estimated
  plot((nd-199:nd),ones(1,200)*thS(1,i),':r')     // true
  set(gca(),'data_bounds',[0 nd -.1 1.1])
  if i==1, 
    title('Evolution ot parameter estimates (left column, only)')
    legend('estimated','true',[350 1.2]);
  end
end

s=nd+1-20:nd;
set(scf(2),'position',[700 60 600 500])
plot(s,y(s),'x:',s,yp(s),'o:','markersize',10)
title 'Prediction'

disp('Simulated parameter',thS)
disp('Estimated parameter',tht)
