// T31preCont.sce
// NP-STEP PREDICTION WITH CONTINUOUS MODEL (KNOWN PARAMETERS)
// - second order regresion model: 
//   y(t)=b0*u(t)+a1*y(t-1)+b1*u(t-1)+a2*y(t-2)+b2*u(t-2)+k+e(t)
// Experiments
// Change: - np = number of steps of prediction
//         - r  = noise variance
//         - th = model parametrs
//         - u  = input signal
// -------------------------------------------------------------------
exec('SCIHOME/ScIntro.sce',-1), mode(0), getd('functions')

nd=100;                            // number of data
np=15;                              // length of prediction (np>=1)
//  b0 a1 b1  a2 b2 k
th=[1 .4 -.3 -.5 .1 1]';           // regression coefficients
r=.2;                             // noise variance
u=sin(4*%pi*(1:nd)/nd)+rand(1,nd,'n'); // input
y=ones(1,2);

yp=ones(1,2);

// TIME LOOP 
for t=3:(nd-np)                    // time loop (on-line tasks)
  // prediction
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]'; // first reg. vec for prediction
  yy=ps'*th;                       // zero prediction at t
  for j=1:np                       // loop of predictions for t+1,...,t+np
    tj=t+j;                        // future times for prediction
    ps=[u(tj); yy; ps(1:$-3); 1];  // reg.vecs with predicted outputs
    yy=ps'*th;                     // new prediction (partial)
  end
  yp(t+np)=yy;                     // final prediction for time t+np
  // simulation
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]';  // regression vector for sim.
  y(t)=ps'*th+sqrt(r)*rand(1,1,'n');      // output generation

end

// Results
s=(np+3):(nd-np);
scf(1);
plot(s,y(s),'.:',s,yp(s),'rx')
set(gca(),"data_bounds",[1 nd -3 5])
legend('output','prediction');
title(string(np)+'-steps ahead prediction')

//Shift of the regression vector
// =============================
//           time t
//ps(t)                             =[u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]
//                                      |    |       |
//           time t+1
//ps(t+1)               =[u(t+1) y(t) u(t) y(t-1) u(t-1)  1]
//                                    ------------------
//                      =[u(t+1) yy(t)    ps(1:$-3)        1]
//                          |     |     |
//           time t+2
//ps(t+2)=[u(t+2) yy(t+1) u(t+1) yy(t) u(t) 1]
//                        -----------------
//       =[t(t+2) yy(t+1)      ps(1:$-3)    1]

