// T32preCont_Adapt.sce
// N-STEP PREDICTION WITH CONTINUOUS MODEL (WITH ESTIMATION)
// - second order regression model: 
//   y(t)=b0*u(t)+a1*y(t-1)+b1*u(t-1)+a2*y(t-2)+b2*u(t-2)+k+e(t)
// Experiments
// Change: - np = number of steps of prediction
//         - r  = noise variance (effect on estimation)
//         - th = model parametrs
//         - u  = input signal (effect on estimation)
// -------------------------------------------------------------------
exec('ScIntro.sce',-1), mode(0), getd('functions')

nd=100;                            // number of data
np=5;                              // length of prediction (np>=1)
//  b0 a1 b1  a2 b2 k
th=[1 .4 -.3 -.5 .1 1]';           // regression coefficients
r=.2;                             // noise variance
u=sin(4*%pi*(1:nd)/nd)+rand(1,nd,'n'); // input
y=ones(1,nd);
Eth=rand(6,1);                     // initial parameter estimates
V=1e-8*eye(7,7);                   // initial information matrix

nu=zeros(4,2);
yp=ones(1,nd);
nz=3;
for t=nz:(nd-np)                   // time loop (on-line tasks)

  // prediction ( without knowledke of y(t) )
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]';       // first reg. vec for prediction
  yy=ps'*Eth;                      // zero prediction at t
  for j=1:np                       // loop of predictions for t+1,...,t+np
    tj=t+j;                        // future times for prediction
    ps=[u(tj); yy; ps(1:$-3); 1];  // reg.vecs with predicted outputs
    yy=ps'*Eth;                    // new prediction (partial)
  end
  yp(t+np)=yy;                     // final prediction for time t+np

  // simulation ( measurement of y(t) )
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]';  // regression vector for sim.
  y(t)=ps'*th+sqrt(r)*rand(1,1,'n');      // output generation
 
  // estimation ( with new data including y(t) )
  Ps=[y(t) u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]';  // reg.vect. for estim.
  V=V+Ps*Ps';                           // update of statistics
  Vp=V(2:$,2:$);
  Vyp=V(2:$,1);
  Eth=inv(Vp+1e-8*eye(Vp))*Vyp;         // point estimates
  Et(:,t)=Eth(:,1);                     // stor est. parameters
end

// Results
disp(' Simulated parameters')
disp(th)
disp(' Estimated parameters')
disp(Eth)

set(scf(1),'position',[100 100 1200 400]);
subplot(121),plot(Et')
set(gca(),"data_bounds",[0 nd+1 -1 2])
title('Evolution of estimated parameters')
subplot(122)
s=(np+2):(nd-np);
plot(s,y(s),'.:',s,yp(s),'rx')
set(gca(),"data_bounds",[1 nd -3 5])
legend('output','prediction');
title([string(np),'-steps ahead prediction'])
