// T33preCat.sce
// PREDICTION WITH DISCRETE MODEL 
//                              u(t),y(t-1) 11 12 21 22
// - known model parameters         y(t)=1   x  x  x  x
// - model: f(y(t)|u(t),y(t-1)      y(t)=2   x  x  x  x
// Experiments
// Change: - np  = number of steps of prediction
//         - th1 = model parametrs
//         - u   = input signal (effect on estimation)
//         - uncertainty of the system (effect on estimation)
// -------------------------------------------------------------------
exec('ScIntro.sce',-1), mode(0), getd('functions')

nd=100;                            // length of data sample
np=0;                              // length of prediction (np>=1)
th1=[0.98 0.01 0.04 0.97];         // parameters for simulation (for y=1)
th=[th1; 1-th1];                   // all parameters
u=(rand(1,nd)>.3)+1;               // input
y=1;

// TIME LOOP
for t=2:(nd-np)
  // prediction
  i=2*(u(t)-1)+y(t-1);             // column of model matrix
  yj=sum(cumsum(th(:,i))<rand(1,1,'u'))+1; // zero-step prediction
  for j=1:np                       // inner loop of prediction
    i=2*(u(t+j)-1)+yj;             // column of model matrix
    yj=sum(cumsum(th(:,i))<rand(1,1,'u'))+1; // prediction
  end
  yp(t+np)=yj;                     // remember the last prediction

// simulation
  i=2*(u(t)-1)+y(t-1);             // column of the table
  y(t)=sum(cumsum(th(:,i))<rand(1,1,'u'))+1;  // output generation
end


// RESULTS
disp(' Model parameters',th), printf('\n')

s=(np+3):(nd-np);
plot(s,y(s),'.:',s,yp(s),'rx')
set(gcf(),'position',[600 100 800 400])
set(gca(),"data_bounds",[0 nd+1 .9 2.1])
legend('output','prediction');
title(string(np)+'-steps ahead prediction')

Wrong=sum(y(s)~=yp(s)), From=nd
