// T34preCat_OffEst.sce
// PREDICTION WITH DISCRETE MODEL (OFF-LINE ESTIMATION)
// Experiments
// Change: - length of prediction
//         - uncertainty of the simulated model
//         - imput signal (effect on estimation)
// -------------------------------------------------------------------
exec('ScIntro.sce',-1), mode(0), getd('functions')

nd=200;                            // number of data
np=3;                              // length of prediction
th1=[0.98 0.01 0.04 0.97];             // parameters for simulation (for y=1)
th=[th1; 1-th1];                   // all parameters
u=(rand(1,nd)>.3)+1;               // input
y=1;

// SIMULATION (measured data)
for t=2:nd
  i=2*(u(t)-1)+y(t-1);             // row of the table
  y(t)=sum(cumsum(th(:,i))<rand(1,1,'u'))+1; // output generation
end

// ESTIMATION (on the whole dataset)
nu=zeros(2,4);
for t=2:nd
  i=2*(u(t)-1)+y(t-1);             // row of the table
  nu(y(t),i)=nu(y(t),i)+1;         // statistics update
end
Eth=nu./(ones(2,1)*sum(nu,1));     // estimate of parameters

// PREDICTION
for t=2:(nd-np)
  i=2*(u(t)-1)+y(t-1);             // column of model matrix
  yj=sum(cumsum(Eth(:,i))<rand(1,1,'u'))+1; // zero-step prediction
  for j=1:np                       // inner loop of prediction
    i=2*(u(t+j)-1)+yj;             // column of model matrix
    yj=sum(cumsum(Eth(:,i))<rand(1,1,'u'))+1; // prediction
  end
  yp(t+np)=yj;                     // remember last prediction
end

// RESULTS
disp(' Simulated parameters')
disp(th)
disp(' Estimated parameters')
disp(Eth)

s=(np+2):(nd-np);
plot(s,y(s),'.:',s,yp(s),'rx')
set(gcf(),'position',[800 100 500 400])
set(gca(),"data_bounds",[0 nd+1 .9 2.1])
legend('output','prediction');
title(string(np)+'-steps ahead prediction')

Wrong=sum(y(s)~=yp(s))
From=nd
