// T35preCat_OnEst.sce
// PREDICTION WITH DISCRETE MODEL (ON-LINE ESTIMATION)
// Change: - length of prediction
//         - uncertainty of the simulated model
//         - imput signal
//         - study the beginning when estimation is not finished
//           how can we secure quicker transient phase of estimation?
// -------------------------------------------------------------------
exec('ScIntro.sce',-1), mode(0), getd('functions')

nd=100;                            // number of data
np=5;                              // length of prediction
th1=[0.98 0.01 0.01 0.98];         // parameters (for y=1)
th=[th1; 1-th1];                   // all parameters
u=(rand(1,nd,'u')>.3)+1;           // input
y=1;

// TIME LOOP
Et=zeros(nd-np,4);
nu=1e-8*rand(2,4,'u');
Eth=nu./(ones(2,1)*sum(nu,1));     // pt estimates;
for t=2:nd-np                      // time loop

  // prediction
  i=2*(u(t)-1)+y(t-1);             // column of model matrix
  yj=sum(cumsum(Eth(:,i))<rand(1,1,'u'))+1; // zero-step prediction
  for j=1:np                       // loop of prediction
    tj=t+j;
    i=2*(u(tj)-1)+yj;              // column of model matrix
    yj=sum(cumsum(Eth(:,i))<rand(1,1,'u'))+1; // prediction
  end
  yp(t+np)=yj;                     // remember last prediction

  // simulation
  i=2*(u(t)-1)+y(t-1);
  y(t)=sum(cumsum(th(:,i))<rand(1,1,'u'))+1; // output
 
  // estimation
  i=2*(u(t)-1)+y(t-1);             // row of model matrix
  nu(y(t),i)=nu(y(t),i)+1;         // statistics update
  Eth=nu./(ones(2,1)*sum(nu,1));   // pt estimates
  Et(t,:)=Eth(1,:);
end

// Results
disp(' Simulated parameters')
disp(th)
disp(' Estimated parameters')
disp(Eth)
title 'Evolution of parameter estimates'

subplot(121),plot(Et)
set(gcf(),'position',[500 100 1000 400])
set(gca(),"data_bounds",[0 nd+1 -.1 1.1])
subplot(122)
s=(np+2):(nd-np);
plot(s,y(s),'.:',s,yp(s),'rx')
set(gca(),"data_bounds",[0 nd+1 .9 2.1])
legend('output','prediction');
title([string(np),'-steps ahead prediction'])

Wrong=sum(y(s)~=yp(s))
From=nd
