// T51ctrlCont.sce
// DYNAMIC PROGRAMMING FOR THE FIRST ORDER RGRESSION MODEL
// Experiments                 ---------------------------
// Change: - regression coefficients a1,b0 and noise se
//         - penalization of input la
// -------------------------------------------------------------------
exec('ScIntro.sce',-1), mode(0), getd('functions')

a1=.8; b0=1;                  // regression coefficients
se=.01;                       // model noise standard deviation
y0=5;                         // initial condition for output
la=.1;                        // penalization of control variable
nd=30;                        // length of control horizon

// SIMULATION WITH NO CONTOL =========================================
y=zeros(1,nd); y(1)=y0; 
for t=2:nd
  u(t)=0;                     // no control
  y(t)=a1*y(t-1)+b0*u(t)+se*rand(1,1,'n'); // without control
end
// ===================================================================

// CONTROL OPTIMIZATION
R=0;                          // initial condition for Bellman function
for t=nd:-1:2                 // runs against the time
  L=la/(1+R);
  S(t)=a1*b0/(b0^2+L);        // coefficient at u
  R=L*a1^2/(b0^2+L);          // partial reminder
end

// CONTROL APPLICATION
yr=zeros(1,nd); yr(1)=y0;     // initial condition for output
ur=zeros(1,nd);
for t=2:nd
  ur(t)=-S(t)*yr(t-1);        // optimal control
  yr(t)=a1*yr(t-1)+b0*ur(t)+se*rand(1,1,'n'); // with opt. control
end

// RESULTS
plot(1:nd,y,'b.:',1:nd,yr,'r.:',1:nd,ur,'g.:')
set(gcf(),"position",[700 100 600 500])
title('Control to zero - dynamic programming, 1st order model')
legend('y - without control','yr - with control','ur - control');
