// T64Mix1Reg.sce
// Mixture estimation (regression components)
// - model  x(t)=k(j)+e(t), j=1,2,3
// - x  scalar
// ------------------------------------------
exec('ScIntro.sce',-1), mode(0), getd('functions')

nd=100;
// Simulation (three components)
thS=[1 3 8];
sdS=[1 1 1]*.5;
alS=[.3 .4 .3];
for t=1:nd
  c(t)=(sum(rand(1,1,'u')>cumsum(alS)))+1;   // pointer
  x(t)=thS(c(t))+sdS(c(t))*rand(1,1,'n');    // data
end

// Initialization
nc=length(thS);
th=[9 2 3];              // initial parameters
ka=[1 1 1];              // counter
al=[1 1 1]/3;            // switching parameter
S=th.*ka;                // statistics for th
ga=al.*ka;               // statistics for al

// Estimation
for t=1:nd
  // classification
  for j=1:nc
    [nill,qL(j)]=GaussN(x(t),th(j),.1); // log-proximity
  end
  qL=qL-max(qL);         // pre-normalization
  q=exp(qL)*al(j);       // proximity
  w=q/sum(q);            // weights

  // statistics update + parameter recomputation
  for j=1:nc
    S(j)=S(j)+w(j)*x(t); // update
    ka(j)=ka(j)+w(j);
    ga(j)=ga(j)+w(j);
  end
  th=S./ka;              // component parameters
  al=ga/sum(ga);         // pointer model parameters
  [nill,pt(t)]=max(w);   // point estimate of the pointer value
  tht(t,:)=th;
end

// Results
z=c2c(c,pt);             // rotation of cp
Acc=acc(c,z(pt))         // accuracy of classification
k=nd-10+1:nd;            // disp classification results
disp('10 last simulated and estimated pointer values')
disp([c(k) pt(k)])
plot(tht)
title 'Evolution of parameter ''th'' estimates'
legend('par1','par2','par3');
