// T64Mix3Cat.sce
// Mixture estimation (binomial componets)
// - model  f(x|c=j), j=1,2,3
// - x  scalar
// ------------------------------------------
exec('ScIntro.sce',-1), mode(0), 
getd c:\functions

nd=200;
// Simulation
N=9;
thS=[.99 .01 .5];
alS=[.3 .5 .2];
for t=1:nd
  c(t)=(sum(rand(1,1,'u')>cumsum(alS)))+1;
  x(t)=grand(1,1,'bin',N,thS(c(t)));
end

// Initialization
nc=length(alS);
th=[.8 .3 .6];
ka=ones(nc,1)*10;
for j=1:nc
  S(:,j)=th(:,j)*ka(j);
end
S0=S;

// Estimation
for t=1:nd
  // classification
  for j=1:nc
    q(j)=binpdf(x(t),th(j),N);
  end
  w=q/sum(q); 
  select 1               // select type of w-transformation
  case 1,
  case 2, w=wtr(w);  
  case 3, w=wdel(w);
  end
  wt(:,t)=w;

  // statistics update + parameter recomputation
  for j=1:nc
    S(j)=S(j)+w(j)*x(t);
    ka(j)=ka(j)+w(j);
    th(:,j)=S(j)/(N*ka(j)); // component parameters
  end
  [nill,pt(t)]=max(w);   // point estimate of the pointer value
  tht(t,:)=th(1,1:3)';
end

// Results
z=c2c(c,pt);             // rotation of cp
Acc=acc(c,z(pt))         // accuracy of classification
k=nd-10+1:nd;            // disp classification results
disp('10 last simulated and estimated pointer values')
disp([c(k) pt(k)])
set(scf(),'position',[800 100 500 400])
plot(tht)
title 'Evolution of component parameter estimates'
legend('p1','p2','p3');
thS
thE=fix(th*100)/100
